/*===========================================================================
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 * 
 * IBM SDK, Java(tm) Technology Edition, v7
 * (C) Copyright IBM Corp. 2013, 2013. All Rights Reserved
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *===========================================================================
 */
/*
 * Copyright (c) 1995, 2011, Oracle and/or its affiliates. All rights reserved.
 * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 */

/**
 * =======================================================================
 * Module Information:
 *
 * DESCRIPTION: This class defines the RDMA network.
 * 
 * This package private class provides RDMA_CAPABILITY.
 * =======================================================================
 */

package java.net;

import java.io.IOException;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.util.List;

class RDMANetworkProvider implements NetworkProvider {
	// native methods
	private native int initialize0();
	private native int cleanup0();
	
	// this provider name
	private static final String name = "RDMA";

	// Preferred addresses
	private InetAddress preferredAddress = null;
	
	static {
		java.security.AccessController.doPrivileged(
				new sun.security.action.LoadLibraryAction("jsor"));
	}

	
	/**
	 * initialize the RDMA network
	 */
	public void initialize() throws IOException {
		if (initialize0() != 0) {
			throw new IOException("Could not initialize RDMA transport!");
		}
	}
	
	/**
	 * destroy RDMA handles that are assigned during
	 * network initialization
	 */
	public void cleanup() {
		cleanup0();
	}
	
	/**
	 * create an RDMA socket impl
	 */
	public SocketImpl createImpl(SocketImpl oldImpl) {
		return new RDMASocketImpl(oldImpl);
	}

	/**
	 * do post-accept operations for an RDMA connection
	 */
	public void postAccept(SocketImpl serverSocketImpl, SocketImpl acceptedSocketImpl) throws IOException {
		long serverSocketDataPtr = ((RDMASocketImpl)serverSocketImpl).getSocketDataPtr();
		long clientSocketDataPtr = ((RDMASocketImpl)serverSocketImpl).socketPostAccept(acceptedSocketImpl, serverSocketDataPtr);
		((RDMASocketImpl)acceptedSocketImpl).setSocketDataPtr(clientSocketDataPtr);
	}

	/**
	 * get the name of this network provider 
	 */
	public String getName() {
		return name;
	}

	/**
	 * Set prefered address based on user preference for RDMA transfer
	 */
	public void setPreferredAddress(List<String> ibAddresses) {
		InetAddress prefAddr = null;
		try {
			if (!ibAddresses.isEmpty()) {
				String preferredAddressStr = AccessController.doPrivileged(new GetPropertyAction("com.ibm.net.rdma.preferredAddress"));
				// preferred address specified is one of the avail IB addresses
				if (preferredAddressStr != null && ibAddresses.contains(preferredAddressStr)) {
					prefAddr = InetAddress.getByName(preferredAddressStr);
				} else {
					// by default we pickup the first avail IB addr
					prefAddr = InetAddress.getByName(ibAddresses.get(0));
 				}
			}
		} catch (IOException e) {
			// let the preferredAddress be null
 		}
 		preferredAddress = prefAddr;
	}

 	/**
	 * Get prefered addresses for RDMA transfer
	 */
 	public InetAddress getPreferredAddress() {
		return preferredAddress;
	}
}
